#ifndef lint
static const char rcsid[] =
	"$Id: utils.c,v 1.3 2001/08/04 14:11:31 andrew Exp $";                                                 
#endif

#include <sys/param.h>
#include <sys/types.h>

#include <paths.h>
#include <stdarg.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <time.h>
#include <unistd.h>

#include "config.h"
#include "utils.h"

char *find_tty(const char *line, unsigned int length) {
	unsigned int length_of_dev,
				 length_of_line;
	char *ret;

	length_of_dev = strlen(_PATH_DEV);
	length_of_line = strnlen(line, length);

	if ((ret = (char *)malloc(sizeof(char) * (length_of_dev + length_of_line + 1))) != NULL) {
		bcopy(_PATH_DEV, ret, length_of_dev);
		bcopy(line, ret + length_of_dev, (size_t)length_of_line);
		ret[length_of_dev + length_of_line] = '\0';
	}

	return(ret);
}

int write_to_tty(const char *tty, const char *const format, ...) {
	FILE *tty_fh;
	char *banner;
	char hostname[MAXHOSTNAMELEN + 1];
	int return_value;
	struct tm *segregate_time;
	time_t current_time;
	va_list ap;

	/* assume failure */
	return_value = 0;

	/* open the tty so we can write to it */
	if ((tty_fh = fopen(tty, "w")) == NULL) {
		syslog(LOG_ERR, "failed to open terminal %s for writing in write_to_tty (%s:%d): %m", tty, __FILE__, __LINE__);
		return(0);
	}

	/* find the current time for the message header */
	if (time(&current_time) == (time_t)-1) {
		syslog(LOG_ERR, "time failed (%s:%d): %m", __FILE__, __LINE__);
		/* not fatal */
		current_time = 0;
	}

	/* break the time up into its components */
	segregate_time = localtime(&current_time);

	/* get the hostname */
	if (gethostname(hostname, sizeof(hostname)) != 0) {
		syslog(LOG_ERR, "gethostname failed (%s:%d): %m", __FILE__, __LINE__);
		/* default to localhost */
		strncpy(hostname, "localhost", sizeof(hostname));
	}

	/* build the banner */
	if (asprintf(&banner, MESSAGE_HEADER, hostname, segregate_time->tm_hour, segregate_time->tm_min) == -1) {
		syslog(LOG_ERR, "asprintf failed (%s:%d): %m", __FILE__, __LINE__);
		/* no banner but we can still try to send the message */
		goto send_message;
	}

	/* send the banner - doesn't really matter if this fails */
	(void)fprintf(tty_fh, "%s", banner);

	/* free banner */
	free(banner);

send_message:
	/* send the message */
	va_start(ap, format);
	if (vfprintf(tty_fh, format, ap) <= 0)  {
		syslog(LOG_ERR, "vfprintf failed (%s:%d): %m", __FILE__, __LINE__);
	} else {
		/* message was sent...at least the first bit */
		return_value = 1;
	}
	va_end(ap);

	/* print EOF so the user knows that's all */
	(void)fprintf(tty_fh, "EOF\r\n");

	/* close the tty device */
	if (fclose(tty_fh) != 0) {
		syslog(LOG_ERR, "fclose failed for %s (%s:%d): %m", tty, __FILE__, __LINE__);
		/* the user may not have seen the mssage after all */
		return_value = 0;
	}

	return(return_value);
}

char *strndup(const char *str, size_t n) {
	char *ret;

	if ((ret = (char *)malloc((sizeof(char) * n) + 1)) != NULL) {
		bcopy((const void *)str, (void *)ret, n);
		ret[n] = '\0';
	}

	return(ret);
}

size_t strnlen(const char *str, size_t n) {
	size_t ret;

	for (ret = 0; (str[ret] != '\0') && (ret < n); ++ret) ;

	return(ret);
}
